package dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.special;

import java.util.function.BiFunction;
import java.util.function.Function;
import com.mojang.datafixers.kinds.Applicative;
import com.mojang.datafixers.kinds.App;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import com.google.common.base.Preconditions;
import net.minecraft.ChatFormatting;
import net.minecraft.util.StringUtil;
import net.minecraft.network.chat.Component;
import dev.shadowsoffire.apotheosis.adventure.affix.Affix;
import net.minecraft.world.damagesource.DamageSource;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.item.ItemStack;
import java.util.Set;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.GemClass;
import com.google.common.collect.ImmutableSet;
import dev.shadowsoffire.apotheosis.adventure.loot.LootCategory;
import dev.shadowsoffire.apotheosis.Apotheosis;
import dev.shadowsoffire.apotheosis.adventure.loot.LootRarity;
import java.util.Map;
import com.mojang.serialization.Codec;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.GemBonus;

public class LeechBlockBonus extends GemBonus
{
    public static Codec<LeechBlockBonus> CODEC;
    protected final Map<LootRarity, Data> values;
    
    public LeechBlockBonus(final Map<LootRarity, Data> values) {
        super(Apotheosis.loc("leech_block"), new GemClass("shield", (Set)ImmutableSet.of(LootCategory.SHIELD)));
        this.values = values;
    }
    
    public float onShieldBlock(final ItemStack gem, final LootRarity rarity, final LivingEntity entity, final DamageSource source, final float amount) {
        final Data d = this.values.get(rarity);
        if (amount <= 2.0f || Affix.isOnCooldown(this.getCooldownId(gem), d.cooldown, entity)) {
            return amount;
        }
        entity.m_5634_(amount * d.healFactor);
        Affix.startCooldown(this.getCooldownId(gem), entity);
        return amount;
    }
    
    public Codec<? extends GemBonus> getCodec() {
        return LeechBlockBonus.CODEC;
    }
    
    public Component getSocketBonusTooltip(final ItemStack gem, final LootRarity rarity) {
        final Data d = this.values.get(rarity);
        final Component cooldown = (Component)Component.m_237110_("affix.apotheosis.cooldown", new Object[] { StringUtil.m_14404_(d.cooldown) });
        return (Component)Component.m_237110_("bonus." + this.getId() + ".desc", new Object[] { Affix.fmt(d.healFactor * 100.0f), cooldown }).m_130940_(ChatFormatting.YELLOW);
    }
    
    public LeechBlockBonus validate() {
        Preconditions.checkNotNull(this.values);
        this.values.forEach((k, v) -> {
            Preconditions.checkNotNull(k);
            Preconditions.checkNotNull(v);
            return;
        });
        return this;
    }
    
    public boolean supports(final LootRarity rarity) {
        return this.values.containsKey(rarity);
    }
    
    public int getNumberOfUUIDs() {
        return 0;
    }
    
    static {
        LeechBlockBonus.CODEC = (Codec<LeechBlockBonus>)RecordCodecBuilder.create(inst -> inst.group((App)LootRarity.mapCodec((Codec)Data.CODEC).fieldOf("values").forGetter(a -> a.values)).apply((Applicative)inst, (Function)LeechBlockBonus::new));
    }
    
    record Data(float healFactor, int cooldown) {
        public static final Codec<Data> CODEC;
        
        static {
            CODEC = RecordCodecBuilder.create(inst -> inst.group((App)Codec.FLOAT.fieldOf("heal_factor").forGetter((Function)Data::healFactor), (App)Codec.INT.fieldOf("cooldown").forGetter((Function)Data::cooldown)).apply((Applicative)inst, (BiFunction)Data::new));
        }
    }
}
